@@ -1,12 +1,13 @@
 import copy
+import sys
 
 from sqlalchemy import util, sql, exc, testing
 from sqlalchemy.testing import assert_raises, assert_raises_message, fixtures
-from sqlalchemy.testing import eq_, is_, ne_, fails_if, mock
+from sqlalchemy.testing import eq_, is_, ne_, fails_if, mock, expect_warnings
 from sqlalchemy.testing.util import picklers, gc_collect
 from sqlalchemy.util import classproperty, WeakSequence, get_callable_argspec
 from sqlalchemy.sql import column
-from sqlalchemy.util import langhelpers
+from sqlalchemy.util import langhelpers, compat
 import inspect
 
 
@@ -342,6 +343,20 @@ class ToListTest(fixtures.TestBase):
             [1, 2, 3]
         )
 
+    def test_from_bytes(self):
+
+        eq_(
+            util.to_list(compat.b('abc')),
+            [compat.b('abc')]
+        )
+
+        eq_(
+            util.to_list([
+                compat.b('abc'), compat.b('def')]),
+            [compat.b('abc'), compat.b('def')]
+        )
+
+
 class ColumnCollectionTest(fixtures.TestBase):
 
     def test_in(self):
@@ -2065,6 +2080,96 @@ class TestClassHierarchy(fixtures.TestBase):
             eq_(set(util.class_hierarchy(A)), set((A, B, object)))
 
 
+class ReraiseTest(fixtures.TestBase):
+    @testing.requires.python3
+    def test_raise_from_cause_same_cause(self):
+        class MyException(Exception):
+            pass
+
+        def go():
+            try:
+                raise MyException("exc one")
+            except Exception as err:
+                util.raise_from_cause(err)
+
+        try:
+            go()
+            assert False
+        except MyException as err:
+            is_(err.__cause__, None)
+
+    def test_reraise_disallow_same_cause(self):
+        class MyException(Exception):
+            pass
+
+        def go():
+            try:
+                raise MyException("exc one")
+            except Exception as err:
+                type_, value, tb = sys.exc_info()
+                util.reraise(type_, err, tb, value)
+
+        assert_raises_message(
+            AssertionError,
+            "Same cause emitted",
+            go
+        )
+
+    def test_raise_from_cause(self):
+        class MyException(Exception):
+            pass
+
+        class MyOtherException(Exception):
+            pass
+
+        me = MyException("exc on")
+
+        def go():
+            try:
+                raise me
+            except Exception:
+                util.raise_from_cause(MyOtherException("exc two"))
+
+        try:
+            go()
+            assert False
+        except MyOtherException as moe:
+            if testing.requires.python3.enabled:
+                is_(moe.__cause__, me)
+
+    @testing.requires.python2
+    def test_safe_reraise_py2k_warning(self):
+        class MyException(Exception):
+            pass
+
+        class MyOtherException(Exception):
+            pass
+
+        m1 = MyException("exc one")
+        m2 = MyOtherException("exc two")
+
+        def go2():
+            raise m2
+
+        def go():
+            try:
+                raise m1
+            except:
+                with util.safe_reraise():
+                    go2()
+
+        with expect_warnings(
+            "An exception has occurred during handling of a previous "
+            "exception.  The previous exception "
+            "is:.*MyException.*exc one"
+        ):
+            try:
+                go()
+                assert False
+            except MyOtherException:
+                pass
+
+
 class TestClassProperty(fixtures.TestBase):
 
     def test_simple(self):
@@ -2082,3 +2187,38 @@ class TestClassProperty(fixtures.TestBase):
         eq_(B.something, {'foo': 1, 'bazz': 2})
 
 
+class TestProperties(fixtures.TestBase):
+
+    def test_pickle(self):
+        data = {'hello': 'bla'}
+        props = util.Properties(data)
+
+        for loader, dumper in picklers():
+            s = dumper(props)
+            p = loader(s)
+
+            eq_(props._data, p._data)
+            eq_(props.keys(), p.keys())
+
+    def test_pickle_immuatbleprops(self):
+        data = {'hello': 'bla'}
+        props = util.Properties(data).as_immutable()
+
+        for loader, dumper in picklers():
+            s = dumper(props)
+            p = loader(s)
+
+            eq_(props._data, p._data)
+            eq_(props.keys(), p.keys())
+
+    def test_pickle_orderedprops(self):
+        data = {'hello': 'bla'}
+        props = util.OrderedProperties()
+        props.update(data)
+
+        for loader, dumper in picklers():
+            s = dumper(props)
+            p = loader(s)
+
+            eq_(props._data, p._data)
+            eq_(props.keys(), p.keys())
